#include <stdarg.h>
#include <stddef.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/timer.h>
#include <linux/config.h>
#include <linux/pci.h>
#include <linux/proc_fs.h>
#include <asm/io.h>

#ifdef CONFIG_COBALT_GEN_III

static unsigned int i2c_io_port;

char *cobalt_temp_read(void);
static char *degrees(int);
static int i2c_read_word(int, int);
static int i2c_wait_for_smi(void);

void __init cobalt_temp_init(void)
{
	struct pci_dev *pmu;

	pmu = pci_find_device(0x10b9, 0x7101, NULL);
	if (!pmu) {
		printk("cobalt_temp_init: no PMU found!\n");
		return;
	}
	
	/* save this */
	pci_read_config_dword(pmu, 0x14, &i2c_io_port);
	i2c_io_port &= 0xfff0;

	printk("Cobalt temperature sensor v1.2 enabled\n");
}

#define I2C_DEV_LM77		0x90
#define LM77_TEMP		0x0
#define LM77_STATUS		0x1
#define LM77_HYST		0x2
#define LM77_CRIT		0x3
#define LM77_LOW		0x4
#define LM77_HIGH		0x5

char *cobalt_temp_read(void)
{
	int tmp;
	int val;

	/* LM77 returns the bytes backwards - <shrug> */
        tmp = i2c_read_word(I2C_DEV_LM77, LM77_TEMP);
        val = (tmp<<8 & 0xff00) + (tmp>>8 & 0x00ff);
	if (!val) {
		/* try again - sometimes the first read is SNAFU */
		tmp = i2c_read_word(I2C_DEV_LM77, LM77_TEMP);
		val = (tmp<<8 & 0xff00) + (tmp>>8 & 0x00ff);
	}
	return degrees(val);
}

/* build a string - measured in degrees C */
static char *degrees(int val)
{
	static char temp[16] = "unknown";

	sprintf(temp, "%d", val>>4);
	if (val & 0x0008) {
		sprintf(temp+strlen(temp), ".5");
	}
	return temp;
}

/* pseudo i2c driver for this one device */
#define I2C_STATUS		0x00
#define I2C_CMD			0x01
#define I2C_START		0x02
#define I2C_ADDR		0x03
#define I2C_LOW_DATA		0x04
#define I2C_HIGH_DATA		0x05
#define I2C_BLOCK_DATA		0x06
#define I2C_INDEX		0x07

#define I2C_STATUS_IDLE		0x04

#define I2C_CMD_RW_WORD		0x30

static int i2c_read_word(int dev, int index)
{
	int ret_val;

	outb(0xff, i2c_io_port + I2C_STATUS); /* clear status */
	outb((dev|0x1) &0xff, i2c_io_port + I2C_ADDR); /* device address */
	outb(I2C_CMD_RW_WORD, i2c_io_port + I2C_CMD); /* Read Write word */
	outb(index&0xff, i2c_io_port + I2C_INDEX); /* I2C index */

	/* issue the start command */
	outb(0xff, i2c_io_port + I2C_START);

	if(i2c_wait_for_smi() < 0)
		return -1;

	ret_val = inb(i2c_io_port + I2C_LOW_DATA) +
		     (inb(i2c_io_port + I2C_HIGH_DATA) << 8);

	return ret_val;
}

static int i2c_wait_for_smi(void)
{
	int timeout=0xffffffff;
	int status;

	while (timeout) {
		outb(0xff, 0x80);  /* wait */
		status = inb(i2c_io_port + I2C_STATUS);  /* get status */
		if (status & I2C_STATUS_IDLE)
			return 0;
		timeout--;
	}

	return -1;
}

#endif /* CONFIG_COBALT_GEN_III */
