/* 
 * Cobalt kernel WDT timer driver
 * Tim Hockin <thockin@cobaltnet.com>
 * Copyright (c)1999, Cobalt Networks
 */
#include <linux/autoconf.h>

#if 0
#if CONFIG_MODVERSIONS == 1 && ! defined(MODVERSIONS)
#define MODVERSIONS
#endif

#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif
#endif

#include <linux/module.h>
#include <stdarg.h>
#include <stddef.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/timer.h>
#include <linux/config.h>
#include <linux/pci.h>

#ifdef CONFIG_COBALT_GEN_III
#include <linux/cobalt_wdt.h>

#define DOGB		0x20
#define ALI_7101_WDT	0x92
#define ALI_WDT_ARM	0x01

struct timer_list cobalt_wdt_timer;
static struct pci_dev *cobalt_pmu;
static char use_pic;

void __init cobalt_wdt_init(void)
{
	struct pci_dev *south;
	char tmp;

#ifdef CONFIG_X86_REMOTE_DEBUG
	printk("Debug kernel - no cobalt watchdog\n");
	return;
#endif
	cobalt_pmu = pci_find_device(PCI_VENDOR_ID_AL, PCI_DEVICE_ID_AL_M7101, 
			NULL);
	if (!cobalt_pmu) {
		printk("cobalt_wdt_init: no PMU found!\n");
		return;
	}

	/* PMU (1543 ver A1-E) has a built-in WDT.  Set it to 1 second */
	pci_write_config_byte(cobalt_pmu, ALI_7101_WDT, 0x02);

	/* reversion # is here - must match ???1001?(b) or use PIC WDT */
	/* why it is called 1543, but DevId is 1533 I'll never know */
	south = pci_find_device(PCI_VENDOR_ID_AL, PCI_DEVICE_ID_AL_M1533, NULL);
	if (!south) {
		printk("cobalt_wdt_init: can't find 1543 device!\n");
		use_pic = 1;
	} else {
		pci_read_config_byte(south, 0x5e, &tmp);
		use_pic = ((tmp & 0x1e) != 0x12);
	}

	if (use_pic) {
		/* set DOGB GPIO pin to OUTPUT - JIC */
		pci_read_config_byte(cobalt_pmu, 0x7d, &tmp);
		pci_write_config_byte(cobalt_pmu, 0x7d, tmp | DOGB);
	}

#ifndef CONFIG_COBALT_BOOTLOADER
	/* set the timer going */
	init_timer(&cobalt_wdt_timer);
	cobalt_wdt_timer.function = cobalt_wdt_refresh;
	cobalt_wdt_timer.data = 1;
	cobalt_wdt_timer.expires = jiffies + WDT_TIMEOUT;
	add_timer(&cobalt_wdt_timer);

	/* the first timer tick will set it going */
	printk("Cobalt watchdog v1.3 enabled");
	if (use_pic) {
		printk(" (old revision board: using PIC controller)");
	}
	printk("\n");
#endif
}

void cobalt_wdt_refresh(unsigned long refresh_timer)
{
	char tmp;

	if (!cobalt_pmu) {
		printk("cobalt_wdt_refresh: no PMU found!\n");
		return;
	}

	/* read the value, disable (reset) WDT, enable WDT */
	pci_read_config_byte(cobalt_pmu, 0x92, &tmp);
	pci_write_config_byte(cobalt_pmu, ALI_7101_WDT, (tmp & ~ALI_WDT_ARM));
	pci_write_config_byte(cobalt_pmu, ALI_7101_WDT, (tmp | ALI_WDT_ARM));

	if (use_pic) {
		/* transition GPIO 5 (DOGB) to arm/clear timer */
		pci_read_config_byte(cobalt_pmu, 0x7e, &tmp);
		pci_write_config_byte(cobalt_pmu, 0x7e, tmp ^ DOGB);
	}
	
	/* re-arm the timer - never called from bootloader */
	if (refresh_timer) {
		mod_timer(&cobalt_wdt_timer, jiffies + WDT_TIMEOUT);
	}
}

/* this is called from machine_restart */
void cobalt_wdt_trigger_reboot(void)
{
	if (!cobalt_pmu) {
		printk("cobalt_wdt_trigger_reboot: no PMU found!\n");
		printk("cobalt_wdt_trigger_reboot: reboot not possible!\n");
		return;
	}

#ifndef CONFIG_COBALT_BOOTLOADER
	/* stop feeding it */
	del_timer(&cobalt_wdt_timer);
#endif

	/* kiss your rear goodbye... */
	cobalt_wdt_refresh(0);
}


EXPORT_SYMBOL(cobalt_wdt_disable);

void cobalt_wdt_disable(void)
{
	char tmp;

#ifdef CONFIG_COBALT_BOOTLOADER
	return;
#endif

	if ( ! cobalt_pmu || use_pic) {
	    if (use_pic)
		printk("cobalt_wdt_disable: in PIC mode - cannot disable\n");
	    return;
	}

	del_timer(&cobalt_wdt_timer);

	/* read the value, disable (reset) WDT */
	pci_read_config_byte(cobalt_pmu, 0x92, &tmp);
	pci_write_config_byte(cobalt_pmu, ALI_7101_WDT, (tmp & ~ALI_WDT_ARM));
}

EXPORT_SYMBOL(cobalt_wdt_reenable);

void cobalt_wdt_reenable(void)
{
#ifdef CONFIG_COBALT_BOOTLOADER
	return;
#endif

	if ( ! cobalt_pmu || use_pic) {
	    return;
	}
	cobalt_wdt_refresh(1);
}

#endif /* CONFIG_COBALT_GEN_III */
